#include "ti86asm.inc"

.org _asm_exec_ram

	nop
	jp ProgStart
	.dw 0
	.dw ShellTitle

ShellTitle:
	.db "Look a moving ship!",0

ProgStart:

	call _runindicoff	;disable run indicator
	call _homeup		;reset screen values (or something...)
	call _clrScrn		;clears Screen
	ld de,Ship1			;loads 1st half of ship sprite
	ld b,50				;set x
	ld c,50				;set y
	push bc				;put bc value in stack (uses this set of bc as it is starting point for left/right/up/down routines)
	call DrawSprite		;draws 1st half of ship
	ld de,Ship2			;loads 2nd half of ship sprite
	ld b,58				;set x
	ld c,50				;set y
	call DrawSprite		;draws 2nd half of ship
	
KeyLoop:
	
	call _getkey		;checks for key pressed
	cp kExit			;if exit then...
	jp z,Exit			;Exit
	cp kLeft			;if left then...
	jp z,Left			;move the ship Left
	cp kRight			;if right then...
	jp z,Right			;move the ship Right
	cp kUp				;if up then...
	jp z,Up				;move the ship Up
	cp kDown			;if down then...
	jp z,Down			;move the ship Down

	jp KeyLoop			;keep repeating this loop

Exit:
	call _homeup		;reset screen values (or something...)
	call _clrScrn		;clears Screen
	pop bc				;remove top byte from stack (keeps from crashing on exit)
	ret					;exit

Right:
	pop bc				;get bc value from stack																	   -|
	push bc				;put bc value into stack (this is done so if b=128 and ship not moved that bc values not lost)	|
						;																								|-all of this is done to make sure ship doesn't loop around screen
	ld a,b				;prepare for 'cp' set a=b																		|
	cp 112				;if a(=b)=112 then ... (112 is the screen width minus the width of sprite)						|	
	jp z,KeyLoop		;return to KeyLoop																			   -|
	
	call _homeup		;reset Screen values (or something...)
	call _clrScrn		;clears Screen

	pop bc				;gets bc value from stack
	inc b				;b=b+1
	inc b				;b=b+1 (done twice to make ship move twice as fast... using inc could be faster than add, not sure)
	push bc				;saves bc value in stack
	ld de,Ship1			;loads 1st half of ship-sprite	
	call DrawSprite		;displays 1st half of ship-sprite
	pop bc				;get bc value from stack
	push bc				;put bc value into stack (using DrawSprite destroys bc variables, as most sprite routines do)
	ld a,8				;a=8
	add a,b				;a=b+a
	ld b,a				;b=a
	ld de,Ship2			;loads 2nd half of ship-sprite
	call DrawSprite		;displays 2nd half of ship-sprite	

	jp KeyLoop

Left:
	pop bc				;get bc value from stack																	   -|
	push bc				;put bc value into stack (this is done so if b=128 and ship not moved that bc values not lost)	|
						;																								|-all of this is done to make sure ship doesn't loop around screen
	ld a,b				;prepare for 'cp' set a=b																		|
	cp 0				;according to most, it should be necissary... but it is											|
	jp z,KeyLoop		;return to KeyLoop																			   -|
	
	call _homeup		;reset Screen values (or something...)
	call _clrScrn		;clears Screen

	pop bc				;gets bc value from stack
	dec b				;b=b-1
	dec b				;b=b-1 (done twice to make ship move twice as fast... using dec could be faster than sub, not sure)
	push bc				;saves bc value in stack
	ld de,Ship1			;loads 1st half of ship-sprite	
	call DrawSprite		;displays 1st half of ship-sprite
	pop bc				;get bc value from stack
	push bc				;put bc value into stack (using DrawSprite destroys bc variables, as most sprite routines do)
	ld a,8				;a=8
	add a,b				;a=b+a
	ld b,a				;b=a
	ld de,Ship2			;loads 2nd half of ship-sprite
	call DrawSprite		;displays 2nd half of ship-sprite	

	jp KeyLoop

Up:
	pop bc				;get bc value from stack																	   -|
	push bc				;put bc value into stack (this is done so if b=128 and ship not moved that bc values not lost)	|
						;																								|-all of this is done to make sure ship doesn't loop around screen
	ld a,c				;prepare for 'cp' set a=c																		|
	cp 0				;according to most, it should be necissary... but it is											|
	jp z,KeyLoop		;return to KeyLoop																			   -|

	
	call _homeup		;reset Screen values (or something...)
	call _clrScrn		;clears Screen

	pop bc				;gets bc value from stack
	dec c				;c=c-1
	dec c				;c=c-1 (done twice to make ship move twice as fast... using dec could be faster than sub, not sure)
	push bc				;saves bc value in stack
	ld de,Ship1			;loads 1st half of ship-sprite	
	call DrawSprite		;displays 1st half of ship-sprite
	pop bc				;get bc value from stack
	push bc				;put bc value into stack (using DrawSprite destroys bc variables, as most sprite routines do)
	ld a,8				;a=8
	add a,b				;a=b+a
	ld b,a				;b=a
	ld de,Ship2			;loads 2nd half of ship-sprite
	call DrawSprite		;displays 2nd half of ship-sprite	

	jp KeyLoop

Down:
	pop bc				;get bc value from stack																	   -|
	push bc				;put bc value into stack (this is done so if b=128 and ship not moved that bc values not lost)	|
						;																								|-all of this is done to make sure ship doesn't loop around screen
	ld a,c				;prepare for 'cp' set a=c																		|
	cp 54				;if ship at this position then... (54=height of screen minus height of sprite minus 2)			|
	jp z,KeyLoop		;return to KeyLoop																			   -|
	
	call _homeup		;reset Screen values (or something...)
	call _clrScrn		;clears Screen

	pop bc				;gets bc value from stack
	inc c				;c=c+1
	inc c				;c=c+1 (done twice to make ship move twice as fast... using dec could be faster than sub, not sure)
	push bc				;saves bc value in stack
	ld de,Ship1			;loads 1st half of ship-sprite	
	call DrawSprite		;displays 1st half of ship-sprite
	pop bc				;get bc value from stack
	push bc				;put bc value into stack (using DrawSprite destroys bc variables, as most sprite routines do)
	ld a,8				;a=8
	add a,b				;a=b+a
	ld b,a				;b=a
	ld de,Ship2			;loads 2nd half of ship-sprite
	call DrawSprite		;displays 2nd half of ship-sprite	

	jp KeyLoop

;#####################
DrawSprite:      ;1997 James Yopp.  If you use it, leave this comment in the source.
        push    hl
        push    de
        push    bc
        call    FP
        ex      de, hl
        ld      b, (hl)
        inc     hl
DS_Loop:
        push    bc
        push    af
        push    af
        ld      b, (hl)
        ld      c, (hl)
        add     a, a
        jr      C, DS_EndB
DS_BLoop:
        SRL     b
        add     a, a
        jr      nc, DS_BLoop
DS_EndB:
        pop     af
DS_CLoop:
        SLA     c
        SRL     a
        jr      nc, DS_CLoop
        inc     hl
        ld      a, (de)
        xor     b
        ld      (de), a
        inc     de
        ld      a, (de)
        xor     c
        ld      (de), a
        push    hl
        ld      hl, 15
        add     hl, de
        ex      de, hl
        pop     hl
        pop     af
        pop     bc
        DJNZ    DS_Loop
        pop     bc
        pop     de        
        pop     hl        
        ret

FP:       ;1997 James Yopp.  If you use it, leave this comment in the source.
        push    de
        push    bc
        push    bc
        ld      h, $FC
        SRL     b
        SRL     b
        SRL     b
        ld      l, b
        ld      b, c
        ld      de, 16
        xor     a              ;A very short way of doing ld a, 0.  This, however,
BLoop:                         ;updates the flags (sometimes a bad thing)
        add     hl, de
        cp b                   ;If I didn't use this stuff, the top display line
        jr      Z, XitLoop     ;wouldn't work.  b would be decreased, leaving 255.
        dec     b              ;If you won't use the top line, change it.
        jr      BLoop
XitLoop:
        pop     bc
        ld      a, b
        and     %00000111
        ld      b, a
        inc     b
        ld      a, %00000001
ALoop:
        RRCA
        DJNZ    ALoop
        pop     bc
        pop     de
        ret

;##########

Ship1:
	.db 8
	.db %00000000
	.db %00000000
	.db %11000011
	.db %11001100
	.db %11110011
	.db %11100010
	.db %11000010
	.db %11111111

Ship2:
	.db 8
	.db %00000000
	.db %00000000
	.db %11000011
	.db %00110011
	.db %11001111
	.db %01000111
	.db %01000011
	.db %11111111

Bullet:
	.db 8	
	.db %00011000
	.db %00011000
	.db %00011000
	.db %00011000
	.db %00011000
	.db %00011000
	.db %00011000
	.db %00011000


.end
